"""
Mail sender Sorbonne Université
Quentin Deschamps, 2020
"""
import tkinter as tk
from mailSenderSU.src.login_interface import LoginInterface
from mailSenderSU.src.mail_interface import MailSenderInterface
from mailSenderSU.src.files import Files
import click
import os


@click.command()
@click.option("--clear",
              help="Clear data : all, user, contacts, history",
              default="")
@click.option("--style", help="Define style : su, polytech",
              default="su")
def main(clear, style):
    """Mail Sender Sorbonne Universite"""
    path = os.path.dirname(os.path.abspath(__file__))
    window = tk.Tk()
    login_interface = LoginInterface(window, style, path)
    window.mainloop()
    if login_interface.login:
        data = Files(path, clear)
        window = tk.Tk()
        MailSenderInterface(window, style, data,
                            login_interface.num_etudiant,
                            login_interface.password)
        window.mainloop()


if __name__ == "__main__":
    main("", "su")



"""
Mail sender Sorbonne Université
Envoi du mail
Quentin Deschamps, 2020
"""
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email import encoders
import mailSenderSU.src.message as Message


def get_name_from_email(email):
    """Retourne le nom associé à une adresse mail"""
    fullname = email.split("@")[0]
    firstname = " ".join([
        i.capitalize() for i in fullname.split(".")[0].split("-")])
    name = " ".join([
        i.capitalize() for i in fullname.split(".")[1].split("_")])
    return f"{firstname} {name}"


def list_email(email):
    """Retourne la liste des emails à partir d'une chaîne de caractères"""
    liste = email.split(",")
    liste = [i.strip(" ") for i in liste]
    return liste


def add_name_to_signature(signature, name, color="#263068"):
    """Ajoute le nom à la signature"""
    signature = signature.split("\n")
    line = f'<p><small><font color={color}>{name}</font></small></p>'
    signature.insert(4, line)
    return "".join(signature)


def is_zip(path):
    """Retourne True si le fichier est un zip"""
    return path.split(".")[-1] == "zip"


def send_mail(email_user, email_send, email_cc, subject,
              body, list_attachment, num_etudiant, password,
              signature, server="smtp.upmc.fr", port=587):
    """Envoie le mail"""
    name = get_name_from_email(email_user)
    msg = MIMEMultipart()
    msg['From'] = f"{name} <{email_user}>"
    msg['To'] = email_send
    if email_cc != "":
        msg['Cc'] = email_cc
    msg['Subject'] = subject

    signature = add_name_to_signature(signature, name)

    msg.attach(MIMEText(body, 'plain'))
    msg.attach(MIMEText(signature, 'html'))

    for path in list_attachment:
        attachment = open(path, 'rb')
        if is_zip(path):
            type_file = 'zip'
        else:
            type_file = 'octet-stream'
        part = MIMEBase('application', type_file)
        part.set_payload(attachment.read())
        encoders.encode_base64(part)
        filename = path.split("/")[-1].replace(" ", "_")
        part.add_header('Content-Disposition',
                        'attachment; filename= ' + filename)

        msg.attach(part)

    text = msg.as_string()
    server = smtplib.SMTP(server, port)
    server.starttls()
    server.login(num_etudiant, password)

    recipients = list_email(email_send)
    if email_cc != "":
        recipients += list_email(email_cc)
    try:
        server.sendmail(email_user, recipients, text)
        server.quit()
    except Exception:
        Message.show_send_mail_error()
        return text
    Message.show_send_mail()
    return text



"""
Mail sender Sorbonne Université
Messages
Quentin Deschamps, 2020
"""
from tkinter.messagebox import showerror, showinfo, askyesno


def show_login_error():
    """Message d'erreur de login"""
    showerror("Erreur", "La connexion a échoué. Veuillez réessayer.")


def show_send_mail():
    """Message d'envoi réussi"""
    showinfo("Envoi réussi", "Le mail a été envoyé !")


def show_email_user_miss():
    """Message d'envoyeur manquant"""
    showerror("Pas d'envoyeur", "Merci d'indiquer votre mail.")


def show_email_send_miss():
    """Message de destinataires manquants"""
    showerror("Pas de destinataires", "Merci d'indiquer les destinataires.")


def show_email_not_valid(mail):
    """Message de mail non valide"""
    showerror("Email non valide", f"L'adresse {mail} n'est pas valide.")


def show_subject_miss():
    """Message d'objet manquant"""
    showerror("Pas d'objet", "Vous avez oublié d'indiquer l'objet !")


def show_new_user(mail):
    """Message de nouvel utilisateur"""
    return askyesno("Nouvelle adresse",
                    f"Ajouter {mail} à vos adresses ?")


def show_new_contact(mail):
    """Message de nouveau contact"""
    return askyesno("Nouveau contact",
                    f"Ajouter {mail} à vos contacts ?")


def show_send_mail_error():
    """Message d'erreur d'envoi"""
    return showerror("Erreur envoi", "L'envoi a échoué !")



"""
Mail sender Sorbonne Université
Gestion des pièces jointes
Quentin Deschamps, 2020
"""
import tkinter as tk
import tkinter.ttk as ttk
from tkinter.filedialog import askopenfilenames


class Attachment:
    """Gestion des pièces jointes"""
    def __init__(self, fenetre, button_frame):
        self.list_attachment = []
        self.fenetre = fenetre
        self.button_frame = button_frame

        # Listbox
        self.frame_listbox = ttk.Frame(self.fenetre)
        self.label_listbox = ttk.Label(self.frame_listbox,
                                       text="Pièces jointes")
        self.listbox_attachment = tk.Listbox(self.frame_listbox,
                                             height=2, width=60)
        self.label_listbox.grid(row=0)
        self.listbox_attachment.grid(row=1)
        self.frame_listbox.grid(row=4, padx=10, pady=10)

        # Button
        self.button_attachment = ttk.Button(self.button_frame,
                                            text="Joindre un fichier",
                                            command=self.open_files)
        self.button_attachment.grid(row=0, column=1, padx=5, pady=5)
        self.button_delete = ttk.Button(self.button_frame,
                                        text="Supprimer",
                                        command=self.delete_file)
        self.button_delete.grid(row=0, column=2, padx=5, pady=5)
        self.button_delete.config(state=tk.DISABLED)

    def open_files(self):
        """Ajoute des pièces jointes"""
        filenames = askopenfilenames(initialdir=".",
                                     title="Ouvrir les fichiers",
                                     filetypes=[('all files', '.*'),
                                                ('pdf files', '.pdf'),
                                                ('zip files', '.zip'),
                                                ('png files', '.png')])
        for f in filenames:
            if f not in self.list_attachment:
                self.list_attachment.append(f)
                self.listbox_attachment.insert(
                    tk.END, f.split("/")[-1].replace(" ", "_"))
                self.button_delete.config(state=tk.NORMAL)

    def delete_file(self):
        """Supprime une pièce jointe"""
        filename = self.listbox_attachment.get(tk.ACTIVE)
        index = self.listbox_attachment.get(0, tk.END).index(filename)
        self.list_attachment.pop(index)
        self.listbox_attachment.delete(index)
        if self.list_attachment == []:
            self.button_delete.config(state=tk.DISABLED)



"""
Mail sender Sorbonne Université
Mail sender interface
Quentin Deschamps, 2020
"""
import tkinter as tk
import tkinter.ttk as ttk
from mailSenderSU.src.send import send_mail
from mailSenderSU.src.attachment import Attachment
from mailSenderSU.src.style import Style
import mailSenderSU.src.message as Message


class MailSenderInterface:
    """Interface du mail sender"""
    def __init__(self, window, style, data, num_etudiant, password):
        self.data = data
        self.num_etudiant = num_etudiant
        self.password = password

        # Fenêtre
        self.fenetre = window
        self.fenetre.title("Mail Sender Sorbonne Université")
        self.fenetre.resizable(width=False, height=False)

        # Style
        self.style = Style(self.fenetre, style, data.path)

        # Partie du haut
        self.frame = ttk.Frame(self.fenetre)
        self.label_email_user = ttk.Label(self.frame, text="De")
        self.entry_email_user = ttk.Combobox(self.frame, width=50,
                                             values=data.list_email_user)
        if data.list_email_user != []:
            self.entry_email_user.current(0)
        self.label_email_user.grid(row=0, column=0, padx=10, pady=5)
        self.entry_email_user.grid(row=0, column=1, padx=10, pady=5)
        self.label_email_send = ttk.Label(self.frame, text="À")
        self.entry_email_send = ttk.Combobox(self.frame, width=50,
                                             values=data.list_email_send)
        self.label_email_send.grid(row=1, column=0, padx=10, pady=5)
        self.entry_email_send.grid(row=1, column=1, padx=10, pady=5)
        self.label_email_cc = ttk.Label(self.frame, text="Cc")
        self.entry_email_cc = ttk.Combobox(self.frame, width=50,
                                           values=data.list_email_send)
        self.label_email_cc.grid(row=2, column=0, padx=10, pady=5)
        self.entry_email_cc.grid(row=2, column=1, padx=10, pady=5)
        self.label_subject = ttk.Label(self.frame, text="Objet")
        self.entry_subject = ttk.Entry(self.frame, width=52)
        self.label_subject.grid(row=3, column=0, padx=10, pady=5)
        self.entry_subject.grid(row=3, column=1, padx=10, pady=5)
        self.frame.grid(row=0, padx=10, pady=10)

        # Corps du texte
        self.entry_body = tk.Text(self.fenetre, height=10, width=60,
                                  wrap=tk.WORD)
        self.entry_body.grid(row=1, padx=10, pady=10)

        # Boutons
        self.button_frame = ttk.Frame(self.fenetre)
        self.button_send = ttk.Button(self.button_frame, text="Envoyer",
                                      command=self.send_message)
        self.button_send.grid(row=0, column=0, padx=5, pady=5)

        # Pièces jointes
        self.attachment = Attachment(self.fenetre, self.button_frame)

        self.button_frame.grid(row=3, padx=10, pady=5)

    def check(self):
        """Vérifie les adresses mail"""
        email_user = self.entry_email_user.get()
        if not email_user:
            Message.show_email_user_miss()
            return False
        email_send = self.entry_email_send.get()
        if not email_send:
            Message.show_email_send_miss()
            return False
        if not ("sorbonne-universite.fr" in email_user
                or "upmc.fr" in email_user):
            Message.show_email_not_valid(email_user)
            return False
        subject = self.entry_subject.get()
        if not subject:
            Message.show_subject_miss()
            return False
        return True

    def send_message(self):
        """Lance l'envoi du mail"""
        if self.check():
            email_user = self.entry_email_user.get()
            email_send = self.entry_email_send.get()
            email_cc = self.entry_email_cc.get()
            subject = self.entry_subject.get()
            body = self.entry_body.get("1.0", "end")
            list_attachment = self.attachment.list_attachment
            if not email_cc:
                email_cc = ""
            text = send_mail(email_user, email_send, email_cc,
                             subject, body, list_attachment,
                             self.num_etudiant, self.password,
                             self.data.signature)
            self.data.maj_files(email_user, email_send, email_cc, text)



"""
Mail sender Sorbonne Université
Style
Quentin Deschamps, 2020
"""
from tkinter import ttk


class Style:
    """Définit le style d'une fenêtre"""
    def __init__(self, window, style, path):
        if style == 'polytech':
            self.image = "".join([path, "/image/polytech.png"])
            bgcolor = '#009ee0'
            fg_button = '#004877'
        else:
            self.image = "".join([path, "/image/su.png"])
            bgcolor = '#263068'
            fg_button = '#e63228'
        theme = 'clam'
        bg_button = '#ffffff'
        fg_label = '#ffffff'
        font_button = ('calibri', 10, 'bold')

        self.window = window
        self.window.configure(bg=bgcolor)
        self.style = ttk.Style()
        if theme in self.style.theme_names():
            self.style.theme_use(theme)
        # Button style
        self.style.configure('TButton',
                             foreground=fg_button,
                             background=bg_button,
                             font=font_button)
        # Label style
        self.style.configure('TLabel', foreground=fg_label, background=bgcolor)
        # Frame style
        self.style.configure('TFrame', background=bgcolor)



"""
Mail sender Sorbonne Université
Gestion des fichiers
Quentin Deschamps, 2020
"""
import mailSenderSU.src.message as Message
import os


def list_file(file):
    """Retourne la liste des éléments d'un fichier"""
    f = open(file, "r")
    lines = f.readlines()
    ret_list = [i.strip("\n") for i in lines]
    f.close()
    return ret_list


def clear_file(file):
    """Supprime les données d'un fichier"""
    f = open(file, "r+")
    f.truncate(0)
    f.close()


def add_file(file, data):
    """Ajoute une donnée à un fichier"""
    f = open(file, "a")
    f.write(f"{data}\n")
    f.close()


def write_file(file, liste_data):
    """Ecrit un fichier à partir d'une liste"""
    f = open(file, "w")
    for data in liste_data:
        f.write(f"{data}\n")
    f.close()


def create_file(file):
    """Crée un fichier vide"""
    with open(file, "w"):
        pass


class Files:
    """Gère les fichiers de données"""
    def __init__(self, path, clear):
        self.path = path
        self.file_user = self.absolute_path("/data/user.txt")
        self.file_contacts = self.absolute_path("/data/contacts.txt")
        self.file_signature = self.absolute_path("/data/signature.html")
        self.file_history = self.absolute_path("/data/history.log")
        self.create_files(clear)
        self.list_email_user = list_file(self.file_user)
        self.list_email_send = list_file(self.file_contacts)
        self.signature = self.get_signature()

    def absolute_path(self, file):
        """Retourne le chemin absolu d'un fichier"""
        return "".join([self.path, file])

    def create_files(self, clear):
        """Crée les fichiers s'ils n'existent pas ou recrée ceux demandés"""
        if (not os.path.exists(self.file_user)
                or clear in ["all", "user"]):
            create_file(self.file_user)
        if (not os.path.exists(self.file_contacts)
                or clear in ["all", "contacts"]):
            create_file(self.file_contacts)
        if (not os.path.exists(self.file_history)
                or clear in ["all", "history"]):
            create_file(self.file_history)

    def get_signature(self):
        """Récupère la signature"""
        f = open(self.file_signature, "r")
        signature = f.read()
        f.close()
        return signature

    def maj_files(self, email_user, email_send, email_cc, text):
        """Met à jour les fichiers de données"""
        add_file(self.file_history, text)
        if email_user not in self.list_email_user:
            if Message.show_new_user(email_user):
                self.list_email_user.append(email_user)
                self.list_email_user.sort()
                write_file(self.file_user, self.list_email_user)
        if email_send not in self.list_email_send:
            if Message.show_new_contact(email_send):
                self.list_email_send.append(email_send)
                self.list_email_send.sort()
                write_file(self.file_contacts, self.list_email_send)
        if email_cc != "" and email_cc not in self.list_email_send:
            if Message.show_new_contact(email_cc):
                self.list_email_send.append(email_cc)
                self.list_email_send.sort()
                write_file(self.file_contacts, self.list_email_send)



"""
Mail sender Sorbonne Université
Login interface
Quentin Deschamps, 2020
"""
import smtplib
import tkinter as tk
import tkinter.ttk as ttk
from mailSenderSU.src.style import Style
from mailSenderSU.src.message import show_login_error


class LoginInterface:
    """Interface de connexion au serveur"""
    def __init__(self, window, style, path):
        self.login = False
        self.num_etudiant, self.password = "", ""

        # Fenêtre
        self.login_window = window
        self.login_window.title("Authentification")
        self.login_window.resizable(width=False, height=False)
        self.login_window.focus_set()
        self.login_window.bind("<Return>", self.return_connect)

        # Style
        self.style = Style(self.login_window, style, path)

        # Gauche
        self.frame_left = ttk.Frame(self.login_window)
        self.frame_login = ttk.Frame(self.frame_left)
        self.label_num_etudiant = ttk.Label(self.frame_login,
                                            text="Numéro étudiant")
        self.label_password = ttk.Label(self.frame_login, text="Mot de passe")
        self.label_num_etudiant.grid(row=0, column=0, padx=5, pady=5)
        self.label_password.grid(row=1, column=0, pady=5)

        self.entry_num_etudiant = ttk.Entry(self.frame_login)
        self.entry_password = ttk.Entry(self.frame_login, show='*')
        self.entry_num_etudiant.grid(row=0, column=1, padx=5, pady=5)
        self.entry_password.grid(row=1, column=1, padx=5, pady=5)
        self.frame_login.grid(row=0, column=0, padx=5, pady=5)

        self.button_login = ttk.Button(self.frame_left, text="Connexion",
                                       command=self.connect)
        self.button_login.grid(row=1, column=0, padx=5, pady=5)
        self.frame_left.grid(row=0, column=0, padx=10, pady=10)

        # Droite
        self.logo = tk.PhotoImage(file=self.style.image)
        self.label_logo = tk.Label(self.login_window, image=self.logo)
        self.label_logo.grid(row=0, column=1, padx=10, pady=10)

    def return_connect(self, event):
        """Connexion au serveur avec la touche Enter"""
        self.connect()

    def connect(self, server="smtp.upmc.fr", port=587):
        """Vérifie la validité du numéro étudiant et du mot de passe"""
        num_etudiant = self.entry_num_etudiant.get()
        password = self.entry_password.get()
        try:
            server = smtplib.SMTP(server, port)
            server.starttls()
            server.login(num_etudiant, password)
            server.quit()
        except Exception:
            show_login_error()
        else:
            self.login = True
            self.num_etudiant = num_etudiant
            self.password = password
            self.login_window.destroy()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mailSenderSU",
    version="0.0.6",
    author="Quentin Deschamps",
    author_email="quentindeschamps18@gmail.com",
    description="Mail sender for Sorbonne Universite",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Quentin18/Mail-Sender-Sorbonne-Universite",
    packages=["mailSenderSU", "mailSenderSU.src",
              "mailSenderSU.image", "mailSenderSU.data"],
    include_package_data=True,
    install_requires=['click'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points="""
    [console_scripts]
    mailSenderSU=mailSenderSU.__main__:main
    """,
)
